/*
 * @(#)Font2DTest.java	1.4 01/02/27
 *
 * Copyright (c) 2000 by Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 * 
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.CheckboxMenuItem;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.MenuBar;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.StringTokenizer;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageDecoder;

/**
 * Font2DTest.java
 *
 * @version @(#)Font2DTest.java	1.2 00/08/22
 * @author Shinsuke Fukuda
 */

/// Main Font2DTest Class

public final class Font2DTest extends Panel implements ActionListener, ItemListener {

    /// Frame that will contain Font2DTest
    private final Frame parent;
    /// FontPanel class that will contain all graphical output
    private final FontPanel fp;
    /// RangeMenu class that contains info about the unicode ranges
    private final RangeMenu rm;

    /// Other menus to set parameters for text drawing
    private final ChoiceV2 fontMenu;
    private final TextField sizeField;
    private final ChoiceV2 styleMenu;
    private final ChoiceV2 textMenu;
    private int currentTextChoice = 0;
    private final ChoiceV2 transformMenu;
    private final ChoiceV2 methodsMenu;
    private final Checkbox useAntialiasCB;
    private final Checkbox useFractionalCB;

    /// CheckboxMenuItems
    private CheckboxMenuItemV2 displayGridCBMI;
    private CheckboxMenuItemV2 force16ColsCBMI;
    private CheckboxMenuItemV2 showFontInfoCBMI;

    /// Dialog boxes
    private Dialog userTextDialog;
    private TextArea userTextArea;
    private Dialog printDialog;
    private Dialog fontInfoDialog;
    private LabelV2 fontInfos[] = new LabelV2[2];
    private FileDialog filePromptDialog;

    private CheckboxGroup printCBGroup;
    private Checkbox printModeCBs[] = new Checkbox[3];

    /// Status bar
    private final LabelV2 statusBar;

    /// Universal font used for labels
    private final Font labelFont = new Font( "dialog", Font.BOLD, 12 );

    /// Initialize GUI variables and its layouts
    public Font2DTest( Frame f, boolean isApplet ) {
        parent = f;
        
        rm = new RangeMenu( this, parent );
        fp = new FontPanel( this, parent );
        statusBar = new LabelV2("");

        fontMenu = new ChoiceV2( this );
        sizeField = new TextField( "12", 3 );
        styleMenu = new ChoiceV2( this );
        textMenu = new ChoiceV2( this );
        transformMenu = new ChoiceV2( this );
        methodsMenu = new ChoiceV2( this );

        useAntialiasCB = new Checkbox( "Antialiasing", false );
        useFractionalCB = new Checkbox(" Fractional Metrics", false );
        useAntialiasCB.setFont( labelFont );
        useFractionalCB.setFont( labelFont );
        sizeField.addActionListener( this );
        useAntialiasCB.addItemListener( this );
        useFractionalCB.addItemListener( this );

        setupPanel();
        setupMenu( isApplet );
        setupDialog();
    }

    /// Set up the main interface panel
    private void setupPanel() {
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.insets = new Insets( 2, 0, 2, 2 );
        this.setLayout( gbl );

        addLabeledComponentToGBL( "Font:", fontMenu, gbl, gbc, this );
        addLabeledComponentToGBL( "Size:", sizeField, gbl, gbc, this );
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        addLabeledComponentToGBL( "Transform:", transformMenu, gbl, gbc, this );
        gbc.gridwidth = 1;

        addLabeledComponentToGBL( "Range:", rm, gbl, gbc, this );
        addLabeledComponentToGBL( "Style:", styleMenu, gbl, gbc, this );
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        addLabeledComponentToGBL( "Text to use:", textMenu, gbl, gbc, this );
        gbc.gridwidth = 1;

        addLabeledComponentToGBL( "Method:", methodsMenu, gbl, gbc, this );
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.CENTER;
        gbl.setConstraints( useAntialiasCB, gbc );
        this.add( useAntialiasCB );
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbl.setConstraints( useFractionalCB, gbc );
        this.add( useFractionalCB );

        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.insets = new Insets( 2, 0, 0, 2 );
        gbc.fill = GridBagConstraints.BOTH;
        gbl.setConstraints( fp, gbc );
        this.add( fp );

        gbc.weighty = 0;
        gbc.insets = new Insets( 0, 2, 0, 0 );
        gbl.setConstraints( statusBar, gbc );
        this.add( statusBar );
    }

    /// Adds a component to a container with a label to its left in GridBagLayout
    private void addLabeledComponentToGBL( String name,
                                           Component c,
                                           GridBagLayout gbl,
                                           GridBagConstraints gbc,
                                           Container target ) {
        LabelV2 l = new LabelV2( name );
        GridBagConstraints gbcLabel = (GridBagConstraints) gbc.clone();
        gbcLabel.insets = new Insets( 2, 2, 2, 0 );
        gbcLabel.gridwidth = 1;
        gbcLabel.weightx = 0;

        if ( c == null )
          c = new Label( "" );

        gbl.setConstraints( l, gbcLabel );
        target.add( l );
        gbl.setConstraints( c, gbc );
        target.add( c );
    }

    /// Sets up menu entries
    private void setupMenu( boolean isApplet ) {
        Menu fileMenu = new Menu( "File" );
        Menu optionMenu = new Menu( "Option" );

        fileMenu.add( new MenuItemV2( "Save Selected Options...", this ));
        fileMenu.add( new MenuItemV2( "Load Options...", this ));
        fileMenu.addSeparator();
        fileMenu.add( new MenuItemV2( "Save as JPEG...", this ));
        fileMenu.add( new MenuItemV2( "Load JPEG File to Compare...", this ));
        fileMenu.add( new MenuItemV2( "Page Setup...", this ));
        fileMenu.add( new MenuItemV2( "Print...", this ));
        fileMenu.addSeparator();
        if ( !isApplet )
          fileMenu.add( new MenuItemV2( "Exit", this ));
        else
          fileMenu.add( new MenuItemV2( "Close", this ));

        displayGridCBMI = new CheckboxMenuItemV2( "Display Grid", true, this );
        force16ColsCBMI = new CheckboxMenuItemV2( "Force 16 Columns", false, this );
        showFontInfoCBMI = new CheckboxMenuItemV2( "Display Font Info", false, this );
        optionMenu.add( displayGridCBMI );
        optionMenu.add( force16ColsCBMI );
        optionMenu.add( showFontInfoCBMI );

        MenuBar mb = parent.getMenuBar();
        if ( mb == null )
          mb = new MenuBar();
        mb.add( fileMenu );
        mb.add( optionMenu );

        parent.setMenuBar( mb );

        String fontList[] =
          GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();

        for ( int i = 0; i < fontList.length; i++ )
          fontMenu.add( fontList[i] );
        fontMenu.select( "Dialog" );

        styleMenu.add( "Plain" );
        styleMenu.add( "Bold" );
        styleMenu.add( "Italic" );
        styleMenu.add( "Bold Italic" );

        transformMenu.add( "None" );
        transformMenu.add( "Scale" );
        transformMenu.add( "Shear" );
        transformMenu.add( "Rotate" );

        methodsMenu.add( "drawString" );
        methodsMenu.add( "drawChars" );
        methodsMenu.add( "drawBytes" );
        methodsMenu.add( "drawGlyphVector" );
        methodsMenu.add( "TextLayout.draw" );
        methodsMenu.add( "GlyphVector.getOutline + draw" );
        methodsMenu.add( "TextLayout.getOutline + draw" );

        textMenu.add( "Unicode Range" );
        textMenu.add( "All Glyphs" );
        textMenu.add( "User Text" );
        textMenu.add( "Text File" );
    }

    /// Sets up the all dialogs used in Font2DTest...
    private void setupDialog() {
        filePromptDialog = new FileDialog( parent );

        /// Prepare user text dialog...
        userTextDialog = new Dialog( parent, "User Text", false );
        Panel dialogTopPanel = new Panel();
        Panel dialogBottomPanel = new Panel();
        LabelV2 message1 = new LabelV2( "Enter text below and then press update" );
        LabelV2 message2 = new LabelV2( "(Unicode char can be denoted by \\uXXXX)" );
        userTextArea = new TextArea( "Java2D!" ) {
            public Dimension getPreferredSize() {
                return new Dimension( 300, 100 );
            }
        };
        ButtonV2 bUpdate = new ButtonV2( "Update", this );
        userTextArea.setFont( new Font( "dialog", Font.PLAIN, 12 ));
        dialogTopPanel.setLayout( new GridLayout( 2, 1 ));
        dialogTopPanel.add( "North", message1 );
        dialogTopPanel.add( "South", message2 );
        dialogBottomPanel.add( bUpdate );

        userTextDialog.setLayout( new BorderLayout() );
        userTextDialog.add( "North", dialogTopPanel );
        userTextDialog.add( "Center", userTextArea );
        userTextDialog.add( "South", dialogBottomPanel );
        userTextDialog.pack();

        /// Prepare printing dialog...
        printCBGroup = new CheckboxGroup();
        printModeCBs[ fp.ONE_PAGE ] =
          new Checkbox( "Print one page from currently displayed character/line",
                        printCBGroup, true );
        printModeCBs[ fp.CUR_RANGE ] =
          new Checkbox( "Print all characters in currently selected range",
                        printCBGroup, false );
        printModeCBs[ fp.ALL_TEXT ] =
          new Checkbox( "Print all lines of text",
                        printCBGroup, false );
        LabelV2 l =
          new LabelV2( "Note: Page range in native \"Print\" dialog will not affect the result" );
        Panel buttonPanel = new Panel();
        printModeCBs[ fp.ALL_TEXT ].setEnabled( false );
        buttonPanel.add( new ButtonV2( "Print", this ));
        buttonPanel.add( new ButtonV2( "Cancel", this ));

        printDialog = new Dialog( parent, "Print...", true );
        printDialog.setResizable( false );
        printDialog.setLayout( new GridLayout( printModeCBs.length + 2, 1 ));
        printDialog.add( l );
        for ( int i = 0; i < printModeCBs.length; i++ ) {
            printModeCBs[i].setFont( labelFont );
            printDialog.add( printModeCBs[i] );
        }
        printDialog.add( buttonPanel );
        printDialog.pack();

        /// Prepare font information dialog...
        fontInfoDialog = new Dialog( parent, "Font info", false );
        fontInfoDialog.setResizable( false );
        fontInfoDialog.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                fontInfoDialog.hide();
                showFontInfoCBMI.setState( false );
            }
        });
        Panel fontInfoPanel = new Panel();
        fontInfoPanel.setLayout( new GridLayout( fontInfos.length, 1 ));
        for ( int i = 0; i < fontInfos.length; i++ ) {
            fontInfos[i] = new LabelV2("");
            fontInfoPanel.add( fontInfos[i] );
        }
        fontInfoDialog.add( fontInfoPanel );

        /// Move the location of the dialog...
        userTextDialog.setLocation( 200, 300 );
        fontInfoDialog.setLocation( 0, 400 );
    }

    /// RangeMenu object signals using this function
    /// when Unicode range has been changed and text needs to be redrawn
    public void fireRangeChanged() {
        int range[] = rm.getSelectedRange();
        fp.setTextToDraw( fp.RANGE_TEXT, range, null, null );
        if ( showFontInfoCBMI.getState() )
          fireUpdateFontInfo();
    }

    /// Changes the message on the status bar
    public void fireChangeStatus( String message, boolean error ) {
        /// If this is not ran as an applet, use own status bar,
        /// Otherwise, use the appletviewer/browser's status bar
        statusBar.setText( message );
        if ( error )
          fp.showingError = true;
        else
          fp.showingError = false;
    }

    /// Updates the information about the selected font
    public void fireUpdateFontInfo() {
        if ( showFontInfoCBMI.getState() ) {
            String infos[] = fp.getFontInfo();
            for ( int i = 0; i < fontInfos.length; i++ )
              fontInfos[i].setText( infos[i] );
            fontInfoDialog.pack();
        }
    }

    /// Displays a file load/save dialog and returns the specified file
    private String promptFile( boolean isSave, String initFileName ) {
        if ( isSave ) {
            filePromptDialog.setMode( FileDialog.SAVE );
            filePromptDialog.setTitle( "Save..." );
        }
        else {
            filePromptDialog.setMode( FileDialog.LOAD );
            filePromptDialog.setTitle( "Load..." );
        }
        
        filePromptDialog.setFile( initFileName );
        filePromptDialog.show();

        String fileName = filePromptDialog.getFile();
        if ( fileName != null )
          return filePromptDialog.getDirectory() + fileName;
        else
          return null;
    }

    /// Converts user text into arrays of String, delimited at newline character
    /// Also replaces any valid escape sequence with appropriate unicode character
    private String[] parseUserText( String orig ) {
        int length = orig.length();
        StringTokenizer perLine = new StringTokenizer( orig, "\n" );
        String textLines[] = new String[ perLine.countTokens() ];
        int lineNumber = 0;

        while ( perLine.hasMoreElements() ) {
            String oneLine = perLine.nextToken();
            int lineLength = oneLine.length();
            int prevEscapeEnd = 0;
            int nextEscape = oneLine.indexOf( "\\u" );
            StringBuffer converted = new StringBuffer();

            while ( nextEscape != -1 ) {
                if ( prevEscapeEnd < nextEscape )
                  converted.append( oneLine.substring( prevEscapeEnd, nextEscape ));
                prevEscapeEnd = nextEscape + 6;
                try {
                    String hex = oneLine.substring( nextEscape + 2, nextEscape + 6 );
                    converted.append( (char) Integer.parseInt( hex, 16 ));
                }
                catch ( Exception e ) {
                    int copyLimit =
                      ( nextEscape + 6 < lineLength ) ? ( nextEscape + 6 ) : lineLength;
                    converted.append( oneLine.substring( nextEscape, copyLimit ));
                }
                nextEscape = oneLine.indexOf( "\\u", prevEscapeEnd );
            }
            if ( prevEscapeEnd < lineLength )
              converted.append( oneLine.substring( prevEscapeEnd, lineLength ));
            textLines[ lineNumber++ ] = converted.toString();
        }
        return textLines;
    }

    /// Reads the text from specified file, detecting UTF-16 encoding
    /// Then breaks the text into String array, delimited at every line break
    private void readTextFile( String fileName ) {
        try {
            String fileText, textLines[];
            BufferedInputStream bis =
              new BufferedInputStream( new FileInputStream( fileName ));
            int numBytes = bis.available();
            if (numBytes == 0) {
                throw new Exception("Text file " + fileName + " is empty");
            }
            byte byteData[] = new byte[ numBytes ];
            bis.read( byteData, 0, numBytes );
            bis.close();

            /// If byte mark is found, then use UTF-16 encoding to convert bytes...
            if (numBytes >= 2 &&
                (( byteData[0] == (byte) 0xFF && byteData[1] == (byte) 0xFE ) ||
                 ( byteData[0] == (byte) 0xFE && byteData[1] == (byte) 0xFF )))
              fileText = new String( byteData, "UTF-16" );
            /// Otherwise, use system default encoding
            else
              fileText = new String( byteData );

            int length = fileText.length();
            StringTokenizer perLine = new StringTokenizer( fileText, "\n" );
            /// Determine "Return Char" used in this file
            /// This simply finds first occurrence of CR, CR+LF or LF...
            for ( int i = 0; i < length; i++ ) {
                char iTh = fileText.charAt( i );
                if ( iTh == '\r' ) {
                    if ( i < length - 1 && fileText.charAt( i + 1 ) == '\n' )
                      perLine = new StringTokenizer( fileText, "\r\n" );
                    else
                      perLine = new StringTokenizer( fileText, "\r" );
                    break;
                }
                else if ( iTh == '\n' )
                  /// Use the one already created
                  break;
            }
            int lineNumber = 0, numLines = perLine.countTokens();
            textLines = new String[ numLines ];

            while ( perLine.hasMoreElements() ) {
                String oneLine = perLine.nextToken();
                if ( oneLine == null )
                  /// To make LineBreakMeasurer to return a valid TextLayout
                  /// on an empty line, simply feed it a space char...
                  oneLine = " ";
                textLines[ lineNumber++ ] = oneLine;
            }
            fp.setTextToDraw( fp.FILE_TEXT, null, null, textLines );
            rm.setEnabled( false );
            methodsMenu.setEnabled( false );
        }
        catch ( Exception ex ) {
            fireChangeStatus( "ERROR: Failed to Read Text File; See Stack Trace", true );
            ex.printStackTrace();
        }
    }

    /// Returns a String storing current configuration
    private void writeCurrentOptions( String fileName ) {
        try {
            String curOptions = fp.getCurrentOptions();
            BufferedOutputStream bos =
              new BufferedOutputStream( new FileOutputStream( fileName ));
            /// Prepend title and the option that is only obtainable here
            int range[] = rm.getSelectedRange();
            String completeOptions =
              ( "Font2DTest Option File\n" + 
                displayGridCBMI.getState() + "\n" +
                force16ColsCBMI.getState() + "\n" +
                showFontInfoCBMI.getState() + "\n" +
                rm.getSelectedItem() + "\n" +
                range[0] + "\n" + range[1] + "\n" + curOptions );

            byte toBeWritten[] = completeOptions.getBytes( "UTF-16" );
            bos.write( toBeWritten, 0, toBeWritten.length );
            bos.close();
        }
        catch ( Exception ex ) {
            fireChangeStatus( "ERROR: Failed to Save Options File; See Stack Trace", true );
            ex.printStackTrace();
        }
    }

    /// Updates GUI visibility/status after some parameters have changed
    private void updateGUI() {
        int selectedText = textMenu.getSelectedIndex();

        /// Set the visibility of User Text dialog
        if ( selectedText == fp.USER_TEXT )
          userTextDialog.show();
        else
          userTextDialog.hide();
        /// Change the visibility/status/availability of Print Dialog buttons
        printModeCBs[ fp.ONE_PAGE ].setState( true );
        if ( selectedText == fp.FILE_TEXT || selectedText == fp.USER_TEXT ) {
            methodsMenu.setEnabled( selectedText == fp.USER_TEXT );
            printModeCBs[ fp.CUR_RANGE ].setEnabled( false );
            printModeCBs[ fp.ALL_TEXT ].setEnabled( true );
        }
        else {
            methodsMenu.setEnabled( selectedText == fp.RANGE_TEXT );
            printModeCBs[ fp.CUR_RANGE ].setEnabled( true );
            printModeCBs[ fp.ALL_TEXT ].setEnabled( false );
        }
        /// Modify RangeMenu and fontInfo label availabilty
        if ( selectedText == fp.RANGE_TEXT ) {
            fontInfos[1].setVisible( true );
            rm.setEnabled( true );
        }
        else {
            fontInfos[1].setVisible( false );
            rm.setEnabled( false );
        }
    }

    /// Loads saved options and applies them
    private void loadOptions( String fileName ) {
        try {
            BufferedInputStream bis =
              new BufferedInputStream( new FileInputStream( fileName ));
            int numBytes = bis.available();
            byte byteData[] = new byte[ numBytes ];
            bis.read( byteData, 0, numBytes );
            bis.close();
            if ( numBytes < 2 || 
		(byteData[0] != (byte) 0xFE || byteData[1] != (byte) 0xFF) )
              throw new Exception( "Not a Font2DTest options file" );
            
            String options = new String( byteData, "UTF-16" );
            StringTokenizer perLine = new StringTokenizer( options, "\n" );
            String title = perLine.nextToken();
            if ( !title.equals( "Font2DTest Option File" ))
              throw new Exception( "Not a Font2DTest options file" );

            /// Parse all options
            boolean displayGridOpt = new Boolean( perLine.nextToken() ).booleanValue();
            boolean force16ColsOpt = new Boolean( perLine.nextToken() ).booleanValue();
            boolean showFontInfoOpt = new Boolean( perLine.nextToken() ).booleanValue();
            String rangeNameOpt = perLine.nextToken();
            int rangeStartOpt = Integer.parseInt( perLine.nextToken() );
            int rangeEndOpt = Integer.parseInt( perLine.nextToken() );
            String fontNameOpt = perLine.nextToken();
            int fontSizeOpt = Integer.parseInt( perLine.nextToken() );
            int fontStyleOpt = Integer.parseInt( perLine.nextToken() );
            int fontTransformOpt = Integer.parseInt( perLine.nextToken() );
            int textToUseOpt = Integer.parseInt( perLine.nextToken() );
            int drawMethodOpt = Integer.parseInt( perLine.nextToken() );
            boolean useAntialiasOpt = new Boolean( perLine.nextToken() ).booleanValue();
            boolean useFractionalOpt = new Boolean( perLine.nextToken() ).booleanValue();
            String userTextOpt[] = { "Java2D!" }, dialogEntry = "Java2D!";
            if ( textToUseOpt == fp.USER_TEXT ) {
                int numLines = perLine.countTokens(), lineNumber = 0;
                if ( numLines != 0 ) {
                    userTextOpt = new String[ numLines ];
                    dialogEntry = "";
                    for ( ; perLine.hasMoreElements(); lineNumber++ ) {
                        userTextOpt[ lineNumber ] = perLine.nextToken();
                        dialogEntry += userTextOpt[ lineNumber ] + "\n";
                    }
                }
            }

            /// Reset GUIs
            displayGridCBMI.setState( displayGridOpt );
            force16ColsCBMI.setState( force16ColsOpt );
            showFontInfoCBMI.setState( showFontInfoOpt );
            rm.setSelectedRange( rangeNameOpt, rangeStartOpt, rangeEndOpt );
            fontMenu.select( fontNameOpt );
            sizeField.setText( String.valueOf( fontSizeOpt ));
            styleMenu.select( fontStyleOpt );
            transformMenu.select( fontTransformOpt );
            textMenu.select( textToUseOpt );
            methodsMenu.select( drawMethodOpt );
            useAntialiasCB.setState( useAntialiasOpt );
            useFractionalCB.setState( useFractionalOpt );
            userTextArea.setText( dialogEntry );
            updateGUI();

            /// Reset option variables and repaint
            fp.loadOptions( displayGridOpt, force16ColsOpt, rangeStartOpt, rangeEndOpt,
                            fontNameOpt, fontSizeOpt, fontStyleOpt, fontTransformOpt,
                            textToUseOpt, drawMethodOpt, useAntialiasOpt,
                            useFractionalOpt, userTextOpt );
            if ( showFontInfoOpt ) {
                fireUpdateFontInfo();
                fontInfoDialog.show();
            }
            else
              fontInfoDialog.hide();
        }
        catch ( Exception ex ) {
            fireChangeStatus( "ERROR: Failed to Load Options File; See Stack Trace", true );
            ex.printStackTrace();
        }
    }

    /// Loads a previously saved image 
    private void loadComparisonJPEG( String fileName ) {
        try {
            BufferedInputStream bis =
              new BufferedInputStream( new FileInputStream( fileName ));
            JPEGImageDecoder jid = JPEGCodec.createJPEGDecoder( bis );
            BufferedImage image = jid.decodeAsBufferedImage();
            bis.close();

            Frame f = new Frame( "Comparison JPEG" );
            ImagePanel ip = new ImagePanel( image );
            f.setResizable( false );
            f.add( ip );
            f.addWindowListener( new WindowAdapter() {
                public void windowClosing( WindowEvent e ) {
                    ( (Frame) e.getSource() ).dispose();
                }
            });
            f.pack();
            f.show();
        }
        catch ( Exception ex ) {
            fireChangeStatus( "ERROR: Failed to Load JPEG File; See Stack Trace", true );
            ex.printStackTrace();
        }
    }

    /// Interface functions...

    /// ActionListener interface function
    /// Responds to MenuItem, TextField and Button actions
    public void actionPerformed( ActionEvent e ) {
        Object source = e.getSource();

        if ( source instanceof MenuItem ) {
            MenuItem mi = (MenuItem) source;
            String itemName = mi.getLabel();

            if ( itemName.equals( "Save Selected Options..." )) {
                String fileName = promptFile( true, "options.txt" );
                if ( fileName != null )
                  writeCurrentOptions( fileName );
            }
            else if ( itemName.equals( "Load Options..." )) {
                String fileName = promptFile( false, "options.txt" );
                if ( fileName != null )
                  loadOptions( fileName );
            }
            else if ( itemName.equals( "Save as JPEG..." )) {
                String fileName = promptFile( true, fontMenu.getSelectedItem() + ".jpeg" );
                if ( fileName != null )
                  fp.doSaveJPEG( fileName );
            }
            else if ( itemName.equals( "Load JPEG File to Compare..." )) {
                String fileName = promptFile( false, null );
                if ( fileName != null )
                  loadComparisonJPEG( fileName );
            }
            else if ( itemName.equals( "Page Setup..." ))
              fp.doPageSetup();
            else if ( itemName.equals( "Print..." ))
              printDialog.show();
            else if ( itemName.equals( "Close" ))
              parent.dispose();
            else if ( itemName.equals( "Exit" ))
              System.exit(0);
        }

        else if ( source instanceof TextField ) {
            TextField tf = (TextField) source;
            if ( tf == sizeField )
              fp.setFontParams( fontMenu.getSelectedItem(),
                                Integer.parseInt(sizeField.getText()),
                                styleMenu.getSelectedIndex(),
                                transformMenu.getSelectedIndex() );
        }

        else if ( source instanceof Button ) {
            String itemName = ( (Button) source ).getLabel();
            /// Print dialog buttons...
            if ( itemName.equals( "Print" )) {
                Checkbox cb = printCBGroup.getSelectedCheckbox();
                for ( int i = 0; i < printModeCBs.length; i++ )
                  if ( printModeCBs[i] == cb ) {
                      printDialog.hide();
                      fp.doPrint( i );
                  }
            }
            else if ( itemName.equals( "Cancel" ))
              printDialog.hide();
            /// Update button from Usert Text Dialog...
            else if ( itemName.equals( "Update" ))
              fp.setTextToDraw( fp.USER_TEXT, null,
                                parseUserText( userTextArea.getText() ), null );
        }
    }

    /// ItemListener interface function
    /// Responds to CheckboxMenuItem, Choice and Checkbox actions
    public void itemStateChanged( ItemEvent e ) {
        Object source = e.getSource();

        if ( source instanceof CheckboxMenuItem ) {
            CheckboxMenuItem cbmi = (CheckboxMenuItem) source;
            if ( cbmi == displayGridCBMI )
              fp.setGridDisplay( displayGridCBMI.getState() );
            else if ( cbmi == force16ColsCBMI )
              fp.setForce16Columns( force16ColsCBMI.getState() );
            else if ( cbmi == showFontInfoCBMI ) {
                if ( showFontInfoCBMI.getState() ) {
                    fireUpdateFontInfo();
                    fontInfoDialog.show();
                }
                else
                  fontInfoDialog.hide();
            }
        }

        else if ( source instanceof Choice ) {
            Choice c = (Choice) source;
            /// RangeMenu handles actions by itself and then calls fireRangeChanged,
            /// so it is not listed or handled here
            if ( c == fontMenu || c == styleMenu || c == transformMenu )
              fp.setFontParams( fontMenu.getSelectedItem(),
                                Integer.parseInt(sizeField.getText()),
                                styleMenu.getSelectedIndex(),
                                transformMenu.getSelectedIndex() );
            else if ( c == methodsMenu )
              fp.setDrawMethod( methodsMenu.getSelectedIndex() );
            else if ( c == textMenu ) {
                int selected = textMenu.getSelectedIndex();

                if ( selected == fp.RANGE_TEXT )
                  fp.setTextToDraw( fp.RANGE_TEXT, rm.getSelectedRange(),
                                    null, null );
                else if ( selected == fp.USER_TEXT )
                  fp.setTextToDraw( fp.USER_TEXT, null,
                                    parseUserText( userTextArea.getText() ), null );
                else if ( selected == fp.FILE_TEXT ) {
                    String fileName = promptFile( false, null );
                    if ( fileName != null )
                      readTextFile( fileName );
                    else {
                        /// User cancelled selection; reset to previous choice
                        c.select( currentTextChoice );
                        return;
                    }
                }
                else if ( selected == fp.ALL_GLYPHS )
                  fp.setTextToDraw( fp.ALL_GLYPHS, null, null, null );

                updateGUI();
                currentTextChoice = selected;
            }
        }

        else if ( source instanceof Checkbox ) {
            fp.setRenderingHints( useAntialiasCB.getState(),
                                  useFractionalCB.getState() );
        }
    }
    
    /// Main function
    public static void main(String argv[]) {
        final Frame f = new Frame( "Font2DTest" );
        final Font2DTest f2dt = new Font2DTest( f, false );
        f.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) { System.exit(0); }
        });

        f.add( f2dt );
        f.pack();
        f.show();
    }

    /// Inner class definitions...

    /// Class to display just an image file
    /// Used to show the comparison JPEG image
    private final class ImagePanel extends Panel {
        private final BufferedImage bi;
        
        public ImagePanel( BufferedImage image ) {
            bi = image;
        }

        public Dimension getPreferredSize() {
            return new Dimension( bi.getWidth(), bi.getHeight() );
        }

        public void paint( Graphics g ) {
            g.drawImage( bi, 0, 0, this );
        }
    }

    /// Classes made to avoid repetitive calls... (being lazy)
    private final class ButtonV2 extends Button {
        public ButtonV2( String name, ActionListener al ) {
            super( name );
            this.setFont( labelFont );
            this.addActionListener( al );
        }
    }

    private final class ChoiceV2 extends Choice {
        public ChoiceV2( ItemListener il ) {
            super();
            this.addItemListener( il );
        }
    }

    private final class LabelV2 extends Label {
        public LabelV2( String name ) {
            super( name );
            this.setFont( labelFont );
        }
    }

    private final class MenuItemV2 extends MenuItem {
        public MenuItemV2( String name, ActionListener al ) {
            super( name );
            this.addActionListener( al );
        }
    }

    private final class CheckboxMenuItemV2 extends CheckboxMenuItem {
        public CheckboxMenuItemV2( String name, boolean b, ItemListener il ) {
            super( name, b );
            this.addItemListener( il );
        }
    }
}
